#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = "indexarray"
diag = True


def makeCorrectTree(vertexsize, cellsize):
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, "Base", 3, 3)
    s = NPY.array([[vertexsize, cellsize, 0]], dtype="int32", order="F")
    z = CGL.newZone(b, "Zone", s, CGK.Unstructured_s)
    g = CGL.newGridCoordinates(z, "GridCoordinates")
    d = CGL.newDataArray(
        g, CGK.CoordinateX_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateY_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateZ_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    tetras = CGL.newElements(
        z,
        "TETRAS",
        CGK.TETRA_4_s,
        NPY.array([[1, cellsize]], "i", order="F"),
        NPY.ones((cellsize * 4), dtype="int32"),
    )
    zbc = CGL.newZoneBC(z)
    return (T, b, z, zbc)


vertexsize = 20
cellsize = 7
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray bad parent"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
i = n[2][0]
z[2].append(CGU.copyNode(i, "PointList"))  # unauthorized parent node
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray bad name"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
i = n[2][0]
i[0] = "ElementList"  # unauthorized name
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray bad data type"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
i = n[2][0]
i[1] = NPY.ones((1, cellsize), dtype="float64", order="F")  # unauthorized data type
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray bad child"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
i = n[2][0]
CGL.newDataArray(i, "{DataArray}")  # unauthorized child
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray element index out of range"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(2, cellsize + 2)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)  # element index out of range
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray element appear several times (Warning)"
diag = True
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [[1, 1, 1, 1, 1, 1, 1]],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)  # values appear several times in list
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray gridlocation vertex"
diag = True
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [[1, 2, 3, 4, vertexsize]],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)  # element index out of range
g = CGL.newGridLocation(n, value=CGK.Vertex_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray gridlocation vertex index out of range"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [[1, 2, 0, 4, vertexsize + 1]],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)  # element index out of range
g = CGL.newGridLocation(n, value=CGK.Vertex_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray gridlocation face"
diag = True
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
ntris = 12
tris = CGL.newElements(
    z,
    "TRIS",
    CGK.TRI_3_s,
    NPY.array([[cellsize + 1, cellsize + ntris]], "i", order="F"),
    NPY.ones((ntris * 3), dtype="int32"),
)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(cellsize + 1, cellsize + ntris + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray gridlocation face index out of range #1"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
ntris = 12
tris = CGL.newElements(
    z,
    "TRIS",
    CGK.TRI_3_s,
    NPY.array([[cellsize + 1, cellsize + ntris]], "i", order="F"),
    NPY.ones((ntris * 3), dtype="int32"),
)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(cellsize + 1, cellsize + ntris + 2)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray gridlocation face index out of range #2"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
ntris = 12
tris = CGL.newElements(
    z,
    "TRIS",
    CGK.TRI_3_s,
    NPY.array([[cellsize + 1, cellsize + ntris]], "i", order="F"),
    NPY.ones((ntris * 3), dtype="int32"),
)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(cellsize, cellsize + ntris + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray InwardNormalList"
diag = True
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
inl = CGL.newIndexArray(n, CGK.InwardNormalList_s, value=NPY.ones([3, cellsize]))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray InwardNormalList bad shape #1"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
inl = CGL.newIndexArray(n, CGK.InwardNormalList_s, value=NPY.ones([2, cellsize]))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray InwardNormalList bad shape #2"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
inl = CGL.newIndexArray(n, CGK.InwardNormalList_s, value=NPY.ones([3, cellsize + 1]))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray InwardNormalList bad shape #3"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
inl = CGL.newIndexArray(n, CGK.InwardNormalList_s, value=NPY.ones([cellsize]))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexarray InwardNormalList bad shape #4"
diag = False
(T, b, z, zbc) = makeCorrectTree(vertexsize, cellsize)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
CGU.nodeDelete(T, CGU.getNodeByPath(n, "BC/" + CGK.PointList_s))
inl = CGL.newIndexArray(n, CGK.InwardNormalList_s, value=NPY.ones([2, cellsize]))
TESTS.append((tag, T, diag))



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  ---------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as C
import CGNS.PAT.cgnserrors as E
import CGNS.PAT.cgnskeywords as K
import numpy as N

data = C.newGoverningEquations(None)
C.newUserDefinedData(data, "{UserDefinedData}")
C.newDescriptor(data, "{Descriptor}")
C.newDiffusionModel(data)
status = "-"
comment = "SIDS GoverningEquations_t"
pattern = [data, status, comment]



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = "flowsolution unstructured"
diag = True


def makeUnstTree(vertexsize, cellsize):
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, "Base", 3, 3)
    s = NPY.array([[vertexsize, cellsize, 0]], dtype="int32", order="F")
    z = CGL.newZone(b, "Zone", s, CGK.Unstructured_s)
    g = CGL.newGridCoordinates(z, "GridCoordinates")
    d = CGL.newDataArray(
        g, CGK.CoordinateX_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateY_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateZ_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    tetras = CGL.newElements(
        z,
        "TETRAS",
        CGK.TETRA_4_s,
        NPY.array([[1, cellsize]], "i", order="F"),
        NPY.ones((cellsize * 4), dtype="int32"),
    )
    return (T, b, z)


vertexsize = 20
cellsize = 7
(T, b, z) = makeUnstTree(vertexsize, cellsize)
sol1 = CGL.newFlowSolution(z, name="sol1", gridlocation=CGK.Vertex_s)
CGL.newDataArray(sol1, "var", value=NPY.ones(vertexsize, dtype="float64", order="F"))
sol2 = CGL.newFlowSolution(z, name="sol2", gridlocation=CGK.CellCenter_s)
CGL.newDataArray(sol2, "var", value=NPY.ones(cellsize, dtype="float64", order="F"))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "flowsolution bad dataarray dim"
diag = False
(T, b, z) = makeUnstTree(vertexsize, cellsize)
sol1 = CGL.newFlowSolution(z, name="sol1", gridlocation=CGK.Vertex_s)
CGL.newDataArray(
    sol1, "var", value=NPY.ones(vertexsize + 1, dtype="float64", order="F")
)
sol2 = CGL.newFlowSolution(z, name="sol2", gridlocation=CGK.CellCenter_s)
CGL.newDataArray(sol2, "var", value=NPY.ones(cellsize + 1, dtype="float64", order="F"))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "flowsolution structured"
diag = True


def makeStTree(vertexsize, cellsize):
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, "{Base}", 3, 3)
    z = CGL.newZone(
        b,
        "{Zone}",
        NPY.array(
            [
                [vertexsize[0], cellsize[0], 0],
                [vertexsize[1], cellsize[1], 0],
                [vertexsize[2], cellsize[2], 0],
            ],
            order="F",
        ),
    )
    g = CGL.newGridCoordinates(z, "GridCoordinates")
    d = CGL.newDataArray(
        g, CGK.CoordinateX_s, NPY.ones(tuple(vertexsize), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateY_s, NPY.ones(tuple(vertexsize), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateZ_s, NPY.ones(tuple(vertexsize), dtype="float64", order="F")
    )
    return (T, b, z)


vertexsize = [5, 7, 9]
cellsize = [i - 1 for i in vertexsize]
(T, b, z) = makeStTree(vertexsize, cellsize)
sol1 = CGL.newFlowSolution(z, name="sol1", gridlocation=CGK.Vertex_s)
CGL.newDataArray(
    sol1, "var", value=NPY.ones(tuple(vertexsize), dtype="float64", order="F")
)
sol2 = CGL.newFlowSolution(z, name="sol2", gridlocation=CGK.CellCenter_s)
CGL.newDataArray(
    sol2, "var", value=NPY.ones(tuple(cellsize), dtype="float64", order="F")
)
sol3 = CGL.newFlowSolution(z, name="sol3", gridlocation=CGK.Vertex_s)
CGL.newRind(sol3, NPY.ones(2 * 3, dtype="int32", order="F"))
dsize = [i + 1 + 1 for i in vertexsize]
CGL.newDataArray(sol3, "var", value=NPY.ones(tuple(dsize), dtype="float64", order="F"))
sol4 = CGL.newFlowSolution(z, name="sol4", gridlocation=CGK.CellCenter_s)
CGL.newRind(sol4, NPY.ones(2 * 3, dtype="int32", order="F"))
dsize = [i + 1 + 1 for i in cellsize]
CGL.newDataArray(sol4, "var", value=NPY.ones(tuple(dsize), dtype="float64", order="F"))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "flowsolution bad dataarray dim"
diag = False
(T, b, z) = makeStTree(vertexsize, cellsize)
sol1 = CGL.newFlowSolution(z, name="sol1", gridlocation=CGK.Vertex_s)
dsize = [i + 1 for i in vertexsize]
CGL.newDataArray(sol1, "var", value=NPY.ones(tuple(dsize), dtype="float64", order="F"))
sol2 = CGL.newFlowSolution(z, name="sol2", gridlocation=CGK.CellCenter_s)
dsize = [i + 1 for i in cellsize]
CGL.newDataArray(sol2, "var", value=NPY.ones(tuple(dsize), dtype="float64", order="F"))
sol3 = CGL.newFlowSolution(z, name="sol3", gridlocation=CGK.Vertex_s)
CGL.newRind(sol3, NPY.ones(2 * 3, dtype="int32", order="F"))
dsize = [i for i in vertexsize]
CGL.newDataArray(sol3, "var", value=NPY.ones(tuple(dsize), dtype="float64", order="F"))
sol4 = CGL.newFlowSolution(z, name="sol4", gridlocation=CGK.CellCenter_s)
CGL.newRind(sol4, NPY.ones(2 * 3, dtype="int32", order="F"))
dsize = [i for i in cellsize]
CGL.newDataArray(sol4, "var", value=NPY.ones(tuple(dsize), dtype="float64", order="F"))
TESTS.append((tag, T, diag))



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------


def run():
    import CGNS.VAL.test.cgu


def run2():
    print("### pyCGNS TEST: starting VAL tests")
    import CGNS.VAL.suite.run

    CGNS.VAL.suite.run.runall()


# --- last line



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  ---------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as C
import CGNS.PAT.cgnserrors as E
import CGNS.PAT.cgnskeywords as K
import numpy as N

# RegionCellDimension
data = [
    "{ZoneSubRegion}",
    N.array([3], dtype=N.int32, order="F"),
    [],
    K.ZoneSubRegion_ts,
]

C.newGridLocation(data, K.Vertex_s)
C.newPointRange(data)
C.newFamilyName(data, "{Family}")
C.newUserDefinedData(data, "{UserDefinedData}")
C.newDescriptor(data, "{Descriptor}")

status = "7.3"
comment = "partial SIDS with some optionals"
pattern = [data, status, comment]



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = "indexrange"
diag = True


def makeCorrectTree(vertexsize, cellsize):
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, "Base", 3, 3)
    s = NPY.array([[vertexsize, cellsize, 0]], dtype="int32", order="F")
    z = CGL.newZone(b, "Zone", s, CGK.Unstructured_s)
    g = CGL.newGridCoordinates(z, "GridCoordinates")
    d = CGL.newDataArray(
        g, CGK.CoordinateX_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateY_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateZ_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    return (T, b, z)


vertexsize = 20
cellsize = 7
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexrange index bad ordered"  # this raises a warning, not an error
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[cellsize, 1]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
# element range not ordered
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexrange bad node shape"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([1, cellsize], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
# ElementRange bad node shape
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexrange on BC_t"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
ntris = 11
tris = CGL.newElements(
    z,
    "TRIS",
    CGK.TRI_3_s,
    NPY.array([[cellsize + 1, cellsize + ntris]], "i", order="F"),
    NPY.ones((ntris * 3), dtype="int32"),
)
n = CGL.newBoundary(
    z,
    "BC",
    [range(cellsize + 1, cellsize + ntris + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
CGU.removeChildByName(n, CGK.PointList_s)
CGU.newNode(
    CGK.PointRange_s,
    NPY.array([[cellsize + 1, cellsize + ntris]], dtype=NPY.int32, order="F"),
    [],
    CGK.IndexRange_ts,
    parent=n,
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexrange on BC_t PointRange index out of range #1"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
ntris = 11
tris = CGL.newElements(
    z,
    "TRIS",
    CGK.TRI_3_s,
    NPY.array([[cellsize + 1, cellsize + ntris]], "i", order="F"),
    NPY.ones((ntris * 3), dtype="int32"),
)
n = CGL.newBoundary(
    z,
    "BC",
    [range(cellsize + 1, cellsize + ntris + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
CGU.removeChildByName(n, CGK.PointList_s)
CGU.newNode(
    CGK.PointRange_s,
    NPY.array([[cellsize + 1, cellsize + ntris + 1]], dtype=NPY.int32, order="F"),
    [],
    CGK.IndexRange_ts,
    parent=n,
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexrange on BC_t PointRange index out of range #2"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
ntris = 11
tris = CGL.newElements(
    z,
    "TRIS",
    CGK.TRI_3_s,
    NPY.array([[cellsize + 1, cellsize + ntris]], "i", order="F"),
    NPY.ones((ntris * 3), dtype="int32"),
)
n = CGL.newBoundary(
    z,
    "BC",
    [range(cellsize + 1, cellsize + ntris + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
CGU.removeChildByName(n, CGK.PointList_s)
CGU.newNode(
    CGK.PointRange_s,
    NPY.array([[cellsize, cellsize + ntris]], dtype=NPY.int32, order="F"),
    [],
    CGK.IndexRange_ts,
    parent=n,
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "indexrange on BC_t ElementRange index out of range"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
ntris = 11
tris = CGL.newElements(
    z,
    "TRIS",
    CGK.TRI_3_s,
    NPY.array([[cellsize + 1, cellsize + ntris]], "i", order="F"),
    NPY.ones((ntris * 3), dtype="int32"),
)
n = CGL.newBoundary(
    z,
    "BC",
    [range(cellsize, cellsize + ntris + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
CGU.removeChildByName(n, CGK.PointList_s)
CGU.newNode(
    CGK.ElementRange_s,
    NPY.array([[cellsize + 1, cellsize + ntris + 1]], dtype=NPY.int32, order="F"),
    [],
    CGK.IndexRange_ts,
    parent=n,
)
TESTS.append((tag, T, diag))
